# coding: utf-8
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import os


class Focus(type):
    """
    Utility class for colorizing output on stdout/err.
    """
    COLORS = {
        'info': '\033[1;37m',
        'okay': '\033[92m',
        'warn': '\033[93m',
        'fail': '\033[91m',
        'repr': '\033[35m',
        'none': '\033[0m'
    }

    @staticmethod
    def init():
        if os.sys.platform == 'win32':
            Focus.disable()

    @staticmethod
    def disable():
        Focus.COLORS = dict((k, '') for k, v in Focus.COLORS.items())

    @staticmethod
    def format(color, msg):
        return '{}{}{}'.format(Focus.COLORS[color], msg, Focus.COLORS['none'])

    @staticmethod
    def fail(msg):
        return Focus.format('fail', msg)

    @staticmethod
    def okay(msg):
        return Focus.format('okay', msg)

    @staticmethod
    def warn(msg):
        return Focus.format('warn', msg)

    @staticmethod
    def repr(msg):
        return Focus.format('repr', msg)

    @staticmethod
    def info(msg):
        return Focus.format('info', msg)


class String(object):
    def __new__(self, u_or_str, encoding="utf-8"):
        return u_or_str if isinstance(u_or_str, str) else u_or_str.decode(encoding)



# coding: utf-8
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import os
import gzip

from email import encoders
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.mime.multipart import MIMEMultipart


class MultipartUserData(object):
    """Combine different types of user-data scripts into a single multipart file."""
    MIME = {
        '#include': 'text/x-include-url',
        '#include-once': 'text/x-include-once-url',
        '#!': 'text/x-shellscript',
        '#cloud-config': 'text/cloud-config',
        '#cloud-config-archive': 'text/cloud-config-archive',
        '#upstart-job': 'text/upstart-job',
        '#part-handler': 'text/part-handler',
        '#cloud-boothook': 'text/cloud-boothook'
    }

    def __init__(self):
        self.container = MIMEMultipart()

    def add(self, path, custom_mime_type="text/plain"):
        maintype, subtype = self.get_mime_type(path, custom_mime_type)
        if maintype == 'text':
            msg = self._add_text(path, subtype)
        else:
            msg = self._add_base(path, maintype, subtype)
        msg.add_header('Content-Disposition', 'attachment', filename=os.path.basename(path))
        self.container.attach(msg)

    def _add_text(self, path, subtype):
        with open(path) as fo:
            return MIMEText(fo.read(), _subtype=subtype)

    def _add_base(self, path, maintype, subtype):
        with open(path, 'rb') as fo:
            msg = MIMEBase(maintype, subtype)
            msg.set_payload(fo.read())
            encoders.encode_base64(msg)
            return msg

    def get_mime_type(self, path, default="text/plain"):
        with open(path, 'rb') as fo:
            line = fo.readline()
        mime_type = default
        for shebang in MultipartUserData.MIME.keys():
            if line.startswith(shebang):
                mime_type = MultipartUserData.MIME[shebang]
                break
        return mime_type.split('/', 1)

    def save(self, path, compress=False):
        with open(path, 'wb') as fo:
            if compress:
                with gzip.GzipFile(fileobj=fo, filename=path) as gz:
                    gz.write(self.container.as_string())
            else:
                fo.write(self.container.as_string())

    def __str__(self):
        return self.container.as_string()



# coding: utf-8
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
"""
Laniakea is a utility for managing EC2 instances at AWS and aids in setting up a fuzzing cluster.
"""
import appdirs
import argparse
import boto.exception
import json
import logging
import os
import re
import shlex
import shutil
import subprocess
import sys

from .core.common import Focus, String
from .core.manager import Laniakea


class LaniakeaCommandLine(object):
    """
    Command-line interface for Laniakea.
    """
    HOME = os.path.dirname(os.path.abspath(__file__))
    VERSION = 0.8

    @classmethod
    def parse_args(cls):
        parser = argparse.ArgumentParser(description='Laniakea Runtime',
                                         prog='laniakea',
                                         add_help=False,
                                         formatter_class=argparse.ArgumentDefaultsHelpFormatter,
                                         epilog='The exit status is 0 for non-failures and 1 for failures.')

        dirs = appdirs.AppDirs("laniakea", "Mozilla Security")
        if not os.path.isdir(dirs.user_config_dir):
            shutil.copytree(os.path.join(cls.HOME, 'examples'), dirs.user_config_dir)

        m = parser.add_argument_group('Mandatory Arguments')

        g = m.add_mutually_exclusive_group(required=True)
        g.add_argument('-create-on-demand', action='store_true', help='Create on-demand instances')
        g.add_argument('-create-spot', action='store_true', help='Create spot instances')
        g.add_argument('-stop', nargs='?', const=-1, metavar='n', help='Stop active instances')
        g.add_argument('-terminate', nargs='?', const=-1, metavar='n', help='Terminate active instances')
        g.add_argument('-status', action='store_true', help='List current state of instances')
        g.add_argument('-run', metavar='cmd', type=str, default='', help='Execute commands via SSH')
        g.add_argument('-list-userdata-macros', action='store_true', help='List available macros')
        g.add_argument('-print-userdata', action='store_true', help='Print the UserData script to stdout')

        u = parser.add_argument_group('UserData Arguments')
        u.add_argument('-userdata', metavar='path', type=argparse.FileType(),
                       default=os.path.join(cls.HOME, 'userdata', 'default.sh'),
                       help='UserData script for cloud-init')
        u.add_argument('-userdata-macros', metavar='k=v', nargs='+', type=str, help='Custom macros')

        o = parser.add_argument_group('Optional Arguments')
        o.add_argument('-tags', metavar='k=v', nargs='+', type=str, help='Assign tags to instances')
        o.add_argument('-only', metavar='k=v', nargs='+', type=str, help='Filter instances')
        o.add_argument('-images', metavar='path', type=argparse.FileType(),
                       default=os.path.join(dirs.user_config_dir, 'images.json'),
                       help='EC2 image definitions')
        o.add_argument('-image-name', metavar='str', type=str, default='default', help='Name of image definition')
        o.add_argument('-image-args', metavar='k=v', nargs='+', type=str, help='Custom image arguments')
        o.add_argument('-profile', metavar='str', type=str, default='laniakea', help='AWS profile name in .boto')
        o.add_argument('-max-spot-price', metavar='#', type=float, default=0.05, help='Max price for spot instances')
        o.add_argument('-region', type=str, default='us-west-2', help='EC2 region')
        o.add_argument('-zone', type=str, default=None, help='EC2 placement zone')
        o.add_argument('-root-device-type', type=str, default='ebs', choices=['ebs', 'instance_store'],
                       help='EC2 placement zone')
        o.add_argument('-ebs-size', type=int, default=None, help='Sets the root disk space size. If unset, the EC2 default is used.')
        o.add_argument('-ebs-volume-type', type=str, default='gp2', choices=['gp2', 'io1', 'standard'],
                       help='Sets the root disk volume type.')
        o.add_argument('-ebs-volume-delete-on-termination', action='store_true', default=False,
                       help='Set this to delete the root EBS volume on termination.')
        o.add_argument('-verbosity', default=2, type=int, choices=list(range(1, 6, 1)),
                       help='Log level for the logging module')
        o.add_argument('-focus', action='store_true', default=False, help=argparse.SUPPRESS)
        o.add_argument('-settings', metavar='path', type=argparse.FileType(),
                       default=os.path.join(dirs.user_config_dir, 'laniakea.json'), help='Laniakea settings')
        o.add_argument('-h', '-help', '--help', action='help', help=argparse.SUPPRESS)
        o.add_argument('-version', action='version', version='%(prog)s {}'.format(cls.VERSION),
                       help=argparse.SUPPRESS)

        return parser.parse_args()

    @staticmethod
    def _convert_pair_to_dict(arg):
        """Utility function which transform k=v strings from the command-line into a dict."""
        return dict(kv.split('=', 1) for kv in arg)

    @staticmethod
    def _convert_str_to_int(arg):
        # FIXME: Convert certain values of keys from images.json to ints.
        for k, v in list(arg.items()):
            try:
                arg[String(k)] = int(v)
            except ValueError as e:
                # Let's assume it is a str and move on.
                pass
        return arg

    @staticmethod
    def list_tags(userdata):
        macros = re.findall("@(.*?)@", userdata)
        logging.info("List of available macros:")
        for m in macros:
            logging.info('\t%r', m)

    @staticmethod
    def handle_tags(userdata, macros):
        macro_vars = re.findall("@(.*?)@", userdata)
        for macro_var in macro_vars:
            if macro_var == "!all_macros_export":
                macro_var_export_list = []
                for defined_macro in macros:
                    macro_var_export_list.append("export %s='%s'" % (defined_macro, macros[defined_macro]))
                macro_var_exports = "\n".join(macro_var_export_list)

                userdata = userdata.replace('@%s@' % macro_var, macro_var_exports)
            elif macro_var not in macros:
                logging.error('Undefined variable @%s@ in UserData script', macro_var)
                return
            else:
                userdata = userdata.replace('@%s@' % macro_var, macros[macro_var])

        return userdata

    @staticmethod
    def handle_import_tags(userdata):
        """Handle @import(filepath)@ tags in a UserData script.

        :param userdata: UserData script content.
        :type userdata: str
        :return: UserData script with the contents of the imported files.
        :rtype: str
        """
        imports = re.findall("@import\((.*?)\)@", userdata)
        if not imports:
            return userdata

        for filepath in imports:
            logging.info('Processing "import" of %s', filepath)
            with open(filepath) as fp:
                content = fp.read()
                userdata = userdata.replace("@import(%s)@" % filepath, content)
        return userdata

    @classmethod
    def main(cls):
        args = cls.parse_args()

        logging.basicConfig(format='[Laniakea] %(asctime)s %(levelname)s: %(message)s',
                            level=args.verbosity * 10,
                            datefmt='%Y-%m-%d %H:%M:%S')

        logging.info('Loading Laniakea configuration from %s', args.settings.name)
        try:
            settings = json.loads(args.settings.read())
        except ValueError as msg:
            logging.error('Unable to parse %s: %s', args.settings.name, msg)
            return 1

        Focus.init() if args.focus else Focus.disable()

        args.only = cls._convert_pair_to_dict(args.only or "")
        args.tags = cls._convert_pair_to_dict(args.tags or "")
        args.image_args = cls._convert_str_to_int(cls._convert_pair_to_dict(args.image_args or {}))

        logging.info('Using image definition "%s" from %s', Focus.info(args.image_name), Focus.info(args.images.name))
        try:
            images = json.loads(args.images.read())
        except ValueError as msg:
            logging.error('Unable to parse %s: %s', args.images.name, msg)
            return 1

        logging.info('Reading user data script content from %s', Focus.info(args.userdata.name))
        userdata = args.userdata.read()
        if args.list_userdata_macros:
            cls.list_tags(userdata)
            return 0
        userdata = cls.handle_import_tags(userdata)

        args.userdata_macros = cls._convert_pair_to_dict(args.userdata_macros or "")
        userdata = cls.handle_tags(userdata, args.userdata_macros)

        if args.print_userdata:
            logging.info("Combined user-data script:")
            print(userdata)
            return 0

        if not userdata:
            return 1

        images[args.image_name]['user_data'] = userdata

        if args.image_args:
            logging.info("Setting custom image parameters for upcoming instances: %r ", args.image_args)
            images[args.image_name].update(args.image_args)

        logging.info('Using Boto configuration profile "%s"', Focus.info(args.profile))

        # If a zone has been specified on the command line, use that for all of our images
        if args.zone:
            for image_name in images:
                images[image_name]['placement'] = args.zone

        cluster = Laniakea(images)
        try:
            cluster.connect(profile_name=args.profile, region=args.region)
        except Exception as msg:
            logging.error(msg)
            return 1

        if args.create_on_demand:
            try:
                cluster.create_on_demand(args.image_name, args.tags, args.root_device_type, args.ebs_size,
                                         args.ebs_volume_type, args.ebs_volume_delete_on_termination)
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1

        if args.create_spot:
            try:
                cluster.create_spot(args.max_spot_price, args.image_name, args.tags, args.root_device_type, args.ebs_size,
                                    args.ebs_volume_type, args.ebs_volume_delete_on_termination)
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1

        if args.stop:
            try:
                cluster.stop(cluster.find(filters=args.only), int(args.stop))
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1

        if args.terminate:
            try:
                cluster.terminate(cluster.find(filters=args.only), int(args.terminate))
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1

        if args.status:
            try:
                for i in cluster.find(filters=args.only):
                    logging.info('%s is %s at %s - tags: %s', i.id, i.state, i.ip_address, i.tags)
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1

        if args.run:
            ssh = settings.get('SSH')
            if not ssh:
                logging.error('No SSH settings defined in %s', args.settings.name)
                return 1

            identity = ssh.get('identity')
            if not identity:
                logging.error('Key for SSH is not defined.')
                return 1
            identity = os.path.expanduser(identity)

            username = ssh.get('username')
            if not username:
                logging.error('User for SSH is not defined.')
                return 1

            logging.info("Bucketing available instances.")
            hosts = []
            try:
                for host in cluster.find(filters=args.only):
                    hosts.append(host)
            except boto.exception.EC2ResponseError as msg:
                logging.error(msg)
                return 1
            logging.info("Executing remote commands on %d instances.", len(hosts))

            # Be able to extend ssh_command from settings.json
            ssh_command = ['ssh',
                           '-o' 'UserKnownHostsFile=/dev/null',
                           '-o' 'StrictHostKeyChecking=no',
                           '-o' 'LogLevel=error',
                           '-i', '%s' % identity]

            for host in hosts:
                command = []
                command.extend(ssh_command)
                command.append('%s@%s' % (username, host.ip_address))
                command.extend(shlex.split(args.run))
                logging.info('Running remote command [%s]: %s', host.ip_address, args.run)
                try:
                    print(subprocess.check_output(command))
                except subprocess.CalledProcessError as msg:
                    logging.error(msg)

        return 0



# coding: utf-8
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
import datetime
import logging
import ssl
import sys
import time


try:
    import boto.ec2
    import boto.exception
except ImportError as msg:
    logging.error(msg)
    sys.exit(-1)

class Laniakea(object):
    """
    Laniakea managing class.
    """

    def __init__(self, images):
        self.ec2 = None
        self.images = images

    def retry_on_ec2_error(self, func, *args, **kwargs):
        """
        Call the given method with the given arguments, retrying if the call
        failed due to an EC2ResponseError. This method will wait at most 30
        seconds and perform up to 6 retries. If the method still fails, it will
        propagate the error.

        :param func: Function to call
        :type func: function
        """
        exception_retry_count = 6
        while True:
            try:
                return func(*args, **kwargs)
            except (boto.exception.EC2ResponseError, ssl.SSLError) as e:
                exception_retry_count -= 1
                if exception_retry_count <= 0:
                    raise e
                time.sleep(5)

    def connect(self, region, **kw_params):
        """Connect to a EC2.

        :param region: The name of the region to connect to.
        :type region: str
        :param kw_params:
        :type kw_params: dict
        """
        self.ec2 = boto.ec2.connect_to_region(region, **kw_params)
        if not self.ec2:
            raise Exception("Unable to connect to region '%s'" % region)

        if self.images:
            # Resolve AMI names in our configuration to their IDs
            logging.info('Retrieving available AMIs...')
            image_names = [img['image_name'] for img in self.images]
            remote_images = self.ec2.get_all_images(owners=['self', 'amazon', 'aws-marketplace'],
                                                    filters={'name': image_names})
            for i in self.images:
                if "image_name" in self.images[i] and 'image_id' not in self.images[i]:
                    image_name = self.images[i]['image_name']
                    for ri in remote_images:
                        if ri.name == image_name:
                            if 'image_id' in self.images[i]:
                                raise Exception("Ambiguous AMI name '%s' resolves to multiple IDs" % image_name)
                            self.images[i]['image_id'] = ri.id
                            del self.images[i]['image_name']
                    if 'image_id' not in self.images[i]:
                        raise Exception("Failed to resolve AMI name '%s' to an AMI ID" % image_name)

    def create_on_demand(self, instance_type='default', tags=None, root_device_type='ebs',
                         size='default', vol_type='gp2', delete_on_termination=False):
        """Create one or more EC2 on-demand instances.

        :param instance_type: A section name in images.json
        :type instance_type: str
        :param tags:
        :type tags: dict
        :return: List of instances created
        :rtype: list
        """
        if root_device_type == 'ebs':
            self.images[instance_type]['block_device_map'] = self._configure_ebs_volume(vol_type, size, delete_on_termination)

        reservation = self.ec2.run_instances(**self.images[instance_type])

        logging.info('Creating requested tags...')
        for i in reservation.instances:
            self.retry_on_ec2_error(self.ec2.create_tags, [i.id], tags or {})

        instances = []
        logging.info('Waiting for instances to become ready...')
        while len(reservation.instances):
            for i in reservation.instances:
                if i.state == 'running':
                    instances.append(i)
                    reservation.instances.pop(reservation.instances.index(i))
                    logging.info('%s is %s at %s (%s)',
                                 i.id,
                                 i.state,
                                 i.public_dns_name,
                                 i.ip_address)
                else:
                    self.retry_on_ec2_error(i.update)
        return instances

    def create_spot_requests(self, price, instance_type='default', root_device_type='ebs',
                             size='default', vol_type='gp2', delete_on_termination=False, timeout=None):
        """Request creation of one or more EC2 spot instances.

        :param price: Max price to pay for spot instance per hour.
        :type price: float
        :param instance_type: A section name in images.json
        :type instance_type: str
        :param timeout: Seconds to keep the request open (cancelled if not fulfilled).
        :type timeout: int
        :return: List of requests created
        :rtype: list
        """
        if root_device_type == 'ebs':
            self.images[instance_type]['block_device_map'] = self._configure_ebs_volume(vol_type, size, delete_on_termination)

        valid_until = None
        if timeout is not None:
            valid_until = (datetime.datetime.now() + datetime.timedelta(seconds=timeout)).isoformat()

        requests = self.ec2.request_spot_instances(price, valid_until=valid_until, **self.images[instance_type])
        return [r.id for r in requests]

    def check_spot_requests(self, requests, tags=None):
        """Check status of one or more EC2 spot instance requests.

        :param requests: List of EC2 spot instance request IDs.
        :type requests: list
        :param tags:
        :type tags: dict
        :return: List of boto.ec2.instance.Instance's created, order corresponding to requests param (None if request still open, boto.ec2.instance.Reservation if request is no longer open)
        :rtype: list
        """
        instances = [None] * len(requests)
        ec2_requests = self.retry_on_ec2_error(self.ec2.get_all_spot_instance_requests, request_ids=requests)

        for req in ec2_requests:
            if req.instance_id:
                instance = None
                instance = self.retry_on_ec2_error(self.ec2.get_only_instances, req.instance_id)[0]

                if not instance:
                    raise Exception("Failed to get instance with id %s for %s request %s" % (req.instance_id, req.status.code, req.id))

                instances[requests.index(req.id)] = instance

                self.retry_on_ec2_error(self.ec2.create_tags, [instance.id], tags or {})
                logging.info('Request %s is %s and %s.',
                             req.id,
                             req.status.code,
                             req.state)
                logging.info('%s is %s at %s (%s)',
                             instance.id,
                             instance.state,
                             instance.public_dns_name,
                             instance.ip_address)
            elif req.state != "open":
                # return the request so we don't try again
                instances[requests.index(req.id)] = req

        return instances

    def cancel_spot_requests(self, requests):
        """Cancel one or more EC2 spot instance requests.

        :param requests: List of EC2 spot instance request IDs.
        :type requests: list
        """
        ec2_requests = self.retry_on_ec2_error(self.ec2.get_all_spot_instance_requests, request_ids=requests)

        for req in ec2_requests:
            req.cancel()

    def create_spot(self, price, instance_type='default', tags=None, root_device_type='ebs',
                    size='default', vol_type='gp2', delete_on_termination=False, timeout=None):
        """Create one or more EC2 spot instances.

        :param price: Max price to pay for spot instance per hour.
        :type price: float
        :param instance_type: A section name in images.json
        :type instance_type: str
        :param tags:
        :type tags: dict
        :return: List of instances created
        :rtype: list
        """
        request_ids = self.create_spot_requests(price, instance_type=instance_type, root_device_type=root_device_type,
                                                size=size, vol_type=vol_type, delete_on_termination=delete_on_termination)
        instances = []
        logging.info("Waiting on fulfillment of requested spot instances.")
        poll_resolution = 5.0
        time_exceeded = False
        while request_ids:
            time.sleep(poll_resolution)

            new_instances = self.check_spot_requests(request_ids, tags=tags)

            if timeout is not None:
                timeout -= poll_resolution
                time_exceeded = timeout <= 0

            fulfilled = []
            for idx, instance in enumerate(new_instances):
                if instance is not None:
                    fulfilled.append(idx)
                if isinstance(instance, boto.ec2.instance.Instance):
                    instances.append(instance)
            for idx in reversed(fulfilled):
                request_ids.pop(idx)

            if request_ids and time_exceeded:
                self.cancel_spot_requests(request_ids)
                break

        return instances

    def _scale_down(self, instances, count):
        """Return a list of |count| last created instances by launch time.

        :param instances: A list of instances.
        :type instances: list
        :param count: Number of instances to scale down.
        :type count: integer
        :return: List of instances to be scaled down.
        :rtype: list
        """
        i = sorted(instances, key=lambda i: i.launch_time, reverse=True)
        if not i:
            return []
        running = len(i)
        logging.info("%d instance/s are running.", running)
        logging.info("Scaling down %d instances of those.", count)
        if count > running:
            logging.info("Scale-down value is > than running instance/s - using maximum of %d!", running)
            count = running
        return i[:count]

    def _configure_ebs_volume(self, vol_type, size, delete_on_termination):
        """Sets the desired root EBS size, otherwise the default EC2 value is used.

        :param vol_type: Type of EBS storage - gp2 (SSD), io1 or standard (magnetic)
        :type vol_type: str
        :param size: Desired root EBS size.
        :type size: int
        :param delete_on_termination: Toggle this flag to delete EBS volume on termination.
        :type delete_on_termination: bool
        :return: A BlockDeviceMapping object.
        :rtype: object
        """
        # From GitHub boto docs: http://git.io/veyDv
        dev_sda1 = boto.ec2.blockdevicemapping.BlockDeviceType()
        dev_sda1.delete_on_termination = delete_on_termination
        dev_sda1.volume_type = vol_type
        if size != 'default':
            dev_sda1.size = size  # change root volume to desired size
        bdm = boto.ec2.blockdevicemapping.BlockDeviceMapping()
        bdm['/dev/sda1'] = dev_sda1
        return bdm

    def stop(self, instances, count=0):
        """Stop each provided running instance.

        :param instances: A list of instances.
        :type instances: list
        """
        if not instances:
            return
        if count > 0:
            instances = self._scale_down(instances, count)
        self.ec2.stop_instances([i.id for i in instances])

    def terminate(self, instances, count=0):
        """Terminate each provided running or stopped instance.

        :param instances: A list of instances.
        :type instances: list
        """
        if not instances:
            return
        if count > 0:
            instances = self._scale_down(instances, count)
        self.ec2.terminate_instances([i.id for i in instances])

    def find(self, instance_ids=None, filters=None):
        """Flatten list of reservations to a list of instances.

        :param instance_ids: A list of instance ids to filter by
        :type instance_ids: list
        :param filters: A dict of |Filter.N| values defined in http://goo.gl/jYNej9
        :type filters: dict
        :return: A flattened list of filtered instances.
        :rtype: list
        """
        instances = []
        reservations = self.retry_on_ec2_error(self.ec2.get_all_instances, instance_ids=instance_ids, filters=filters)
        for reservation in reservations:
            instances.extend(reservation.instances)
        return instances



#!/usr/bin/env python
# coding=utf-8
#
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at https://mozilla.org/MPL/2.0/.
from setuptools import setup

if __name__ == '__main__':
    setup(
        classifiers=[
            'Intended Audience :: Developers',
            'Topic :: Software Development :: Testing',
            'Topic :: Security',
            'License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)',
            'Programming Language :: Python :: 2',
            'Programming Language :: Python :: 2.7',
            'Programming Language :: Python :: 3',
            'Programming Language :: Python :: 3.4',
            'Programming Language :: Python :: 3.5',
            'Programming Language :: Python :: 3.6'
        ],
        description='Utility to automate creating EC2 instances.',
        entry_points={
            "console_scripts": ["laniakea = laniakea:LaniakeaCommandLine.main"]
        },
        install_requires=['appdirs', 'boto'],
        license='MPL 2.0',
        maintainer='Christoph Diehl',
        maintainer_email='cdiehl@mozilla.com',
        name='laniakea',
        packages=['laniakea', 'laniakea.core'],
        package_data={'laniakea': [
            'examples/*',
            'userdata/*',
        ]},
        url='https://github.com/MozillaSecurity/laniakea',
        version='0.8')
